"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xslLexer_1 = require("./xslLexer");
const dcpSchema_1 = require("./dcpSchema");
const xsltSchema_1 = require("./xsltSchema");
const dcpSnippets_1 = require("./dcpSnippets");
const xsltSnippets_1 = require("./xsltSnippets");
const xmlSnippets_1 = require("./xmlSnippets");
const schSchema_1 = require("./schSchema");
class XSLTConfiguration {
}
exports.XSLTConfiguration = XSLTConfiguration;
// Note: Non-standard 'else', 'then', 'on-duplicates' can be used in Saxon 10.0
XSLTConfiguration.expressionAtts = ['context-item', 'count', 'else', 'from', 'group-adjacent', 'group-by', 'group-ending-with', 'group-starting-with', 'from', 'for-each-item', 'for-each-source', 'initial-value',
    'key', 'match', 'namespace-context', 'on-duplicates', 'select', 'test', 'then', 'use', 'use-when', 'value', 'with-params', 'xpath'];
XSLTConfiguration.avtAtts = ['allow-duplicate-names', 'base-uri', 'build-tree', 'byte-order-mark', 'case-order', 'cdata-section-elements', 'collation', 'data-type', 'doctype-public', 'doctype-system', 'encoding', 'error-code',
    'escape-uri-attributes', 'flags', 'format', 'grouping-separator', 'grouping-size', 'href', 'html-version', 'include-context-type', 'indent', 'item-separator', 'json-node-output-method',
    'lang', 'letter-value', 'media-type', 'method', 'name', 'namespace', 'normalization-form', 'omit-xml-declaration', 'order', 'ordinal', 'ordinal-type', 'output-version',
    'parameter-document', 'regex', 'separator', 'schema-aware', 'stable', 'standalone', 'suppress-indentaion', 'terminate', 'undeclar-prefixes', 'start-at'];
XSLTConfiguration.xsltPrefix = 'xsl';
XSLTConfiguration.configuration = {
    expressionAtts: XSLTConfiguration.expressionAtts,
    variableElementNames: ['xsl:variable', 'xsl:param'],
    avtAtts: XSLTConfiguration.avtAtts,
    nativePrefix: XSLTConfiguration.xsltPrefix,
    tvtAttributes: ['expand-text'],
    nonNativeAvts: true,
    rootElementSnippets: xsltSnippets_1.XSLTSnippets.xsltRootTags,
    schemaData: new xsltSchema_1.XSLTSchema(),
    docType: xslLexer_1.DocumentTypes.XSLT
};
class XPathConfiguration {
}
exports.XPathConfiguration = XPathConfiguration;
XPathConfiguration.configuration = {
    expressionAtts: [],
    variableElementNames: [],
    nativePrefix: '',
    tvtAttributes: [],
    nonNativeAvts: true,
    rootElementSnippets: [],
    docType: xslLexer_1.DocumentTypes.XPath
};
class DCPConfiguration {
}
exports.DCPConfiguration = DCPConfiguration;
// initial configuration is for basic XProc support only
DCPConfiguration.configuration = {
    expressionAtts: ['xpath', 'when'],
    variableElementNames: ['stringParameter', 'booleanParameter'],
    linkElementAttrNames: ['file', 'path'],
    nativePrefix: '',
    tvtAttributes: [],
    nonNativeAvts: false,
    schemaData: new dcpSchema_1.DCPSchema(),
    docType: xslLexer_1.DocumentTypes.DCP,
    rootElementSnippets: dcpSnippets_1.DCPSnippets.xsltRootTags,
    resourceNames: [
        'xsl/apply-ignore-changes.xsl',
        'xsl/attribute-ignore-changes-marker.xsl',
        'xsl/clean-house.xsl',
        'xsl/clean-non-delta-dxml-content.xsl',
        'xsl.propagate-ignore-changes.xsl',
        'xsl/schema-input-filter.xsl',
        'xsl/extract-format-version.xsl',
        'xsl/split-adjacent-ignorable-whitespace.xsl'
    ],
    featureNames: [
        'http://apache.org/xml/features/validation/schema',
        'http://apache.org/xml/features/nonvalidating/load-dtd-grammar',
        'http://apache.org/xml/features/nonvalidating/load-external-dtd',
        'http://apache.org/xml/features/validation/dynamic',
        'http://apache.org/xml/features/validation/schema'
    ],
    propertyNames: [
        'http://apache.org/xml/properties/schema/external-schemaLocation',
        'http://apache.org/xml/properties/schema/external-noNamespaceSchemaLocation',
    ]
};
class SchConfiguration {
}
exports.SchConfiguration = SchConfiguration;
// initial configuration is for basic XProc support only
SchConfiguration.configuration = {
    expressionAtts: ['context', 'test', 'subject', 'path', 'select', 'value'],
    variableElementNames: ['stringParameter', 'booleanParameter'],
    linkElementAttrNames: ['file', 'path'],
    nativePrefix: '',
    tvtAttributes: [],
    nonNativeAvts: false,
    schemaData: new schSchema_1.SchSchema(),
    docType: xslLexer_1.DocumentTypes.SCH,
    rootElementSnippets: dcpSnippets_1.DCPSnippets.xsltRootTags
};
class XProcConfiguration {
}
exports.XProcConfiguration = XProcConfiguration;
// initial configuration is for basic XProc support only
XProcConfiguration.configuration = {
    expressionAtts: ['select', 'test'],
    variableElementNames: ['p:variable', 'p:param'],
    nativePrefix: 'p',
    tvtAttributes: ['expand-text', 'inline-expand-text'],
    nonNativeAvts: true,
    docType: xslLexer_1.DocumentTypes.Other
};
class XMLConfiguration {
}
exports.XMLConfiguration = XMLConfiguration;
XMLConfiguration.configuration = {
    expressionAtts: [],
    variableElementNames: [],
    nativePrefix: 'qz',
    tvtAttributes: [],
    nonNativeAvts: false,
    rootElementSnippets: xmlSnippets_1.XMLSnippets.xsltRootTags,
    elementSnippets: xmlSnippets_1.XMLSnippets.generalTags,
    docType: xslLexer_1.DocumentTypes.Other
};
class XSLTLightConfiguration {
}
exports.XSLTLightConfiguration = XSLTLightConfiguration;
// used for global instruction processing only
XSLTLightConfiguration.configuration = {
    expressionAtts: [],
    variableElementNames: [],
    nativePrefix: 'xsl',
    tvtAttributes: [],
    nonNativeAvts: false,
    docType: xslLexer_1.DocumentTypes.XSLT
};
//# sourceMappingURL=languageConfigurations.js.map