"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *  Copyright (c) 2020 DeltaXML Ltd. and others.
 *
 *  Contributors:
 *  DeltaXML Ltd. - saxonTaskProvider
 */
const vscode = require("vscode");
const documentChangeHandler_1 = require("./documentChangeHandler");
const jsc = require("jsonc-parser");
class SaxonJsTaskProvider {
    constructor(workspaceRoot) {
        this.workspaceRoot = workspaceRoot;
        this.templateTaskLabel = 'Saxon-JS Transform (New)';
        this.templateTaskFound = false;
    }
    provideTasks() {
        return __awaiter(this, void 0, void 0, function* () {
            const tasksObject = yield SaxonJsTaskProvider.getTasksObject();
            return this.getTasks(tasksObject.tasks);
        });
    }
    resolveTask(_task) {
        return this.getTask(_task.definition);
    }
    static getTasksObject() {
        return __awaiter(this, void 0, void 0, function* () {
            let workspaceUri = vscode.workspace.workspaceFolders ? vscode.workspace.workspaceFolders[0].uri : vscode.Uri.file('/');
            let workspaceTaskUri = workspaceUri.with({ path: workspaceUri.path + '/.vscode/tasks.json' });
            let tasksObject = undefined;
            try {
                const doc = yield vscode.workspace.openTextDocument(workspaceTaskUri);
                tasksObject = jsc.parse(doc.getText());
            }
            catch (e) {
                tasksObject = { tasks: [] };
            }
            return tasksObject;
        });
    }
    getProp(obj, prop) {
        return obj[prop];
    }
    getTasks(tasks) {
        let result = [];
        this.templateTaskFound = false;
        tasks.forEach((task) => {
            let newTask = this.getTask(task);
            if (newTask) {
                result.push(newTask);
            }
        });
        if (!this.templateTaskFound) {
            let templateTask = this.addTemplateTask();
            if (templateTask) {
                result.push(templateTask);
            }
        }
        return result;
    }
    addTemplateTask() {
        let xmlSourceValue = '${file}';
        let xsltFilePath = '${file}';
        let resultPathValue = '${workspaceFolder}/xslt-out/result1.xml';
        let xsltTask = {
            type: 'xslt-js',
            label: this.templateTaskLabel,
            xsltFile: xsltFilePath,
            xmlSource: xmlSourceValue,
            resultPath: resultPathValue,
            group: {
                kind: "build"
            }
        };
        return this.getTask(xsltTask);
    }
    getTask(genericTask) {
        let source = 'xslt-js';
        if (genericTask.type === 'xslt-js') {
            let xsltTask = genericTask;
            if (xsltTask.label === 'xslt-js: ' + this.templateTaskLabel) {
                this.templateTaskFound = true;
            }
            let npxCommand = documentChangeHandler_1.DocumentChangeHandler.isWindowsOS ? 'npx.cmd' : 'npx';
            let commandLineArgs = ['xslt3'];
            let xsltParameters = xsltTask.parameters ? xsltTask.parameters : [];
            let xsltParametersCommand = [];
            let useJSON = !!xsltTask.useJsonSource;
            let nogo = xsltTask.execute !== undefined && xsltTask.execute === false;
            for (const param of xsltParameters) {
                xsltParametersCommand.push(param.name + '=' + param.value);
            }
            for (const propName in xsltTask) {
                let propValue = this.getProp(xsltTask, propName);
                let propNameValue = '';
                switch (propName) {
                    case 'xsltFile':
                        propNameValue = '-xsl:' + propValue;
                        break;
                    case 'xmlSource':
                        const prefix = useJSON ? '-json:' : '-s:';
                        if (propValue !== "") {
                            propNameValue = prefix + propValue;
                        }
                        break;
                    case 'jsonSource':
                        propNameValue = '-json:' + propValue;
                        break;
                    case 'resultPath':
                        propNameValue = '-o:' + propValue;
                        break;
                    case 'initialTemplate':
                        propNameValue = propValue.length === 0 ? '-it' : '-it:' + propValue;
                        break;
                    case 'initialMode':
                        propNameValue = '-im:' + propValue;
                        break;
                    case 'export':
                        propNameValue = '-export:' + propValue;
                        // TODO: add '-nogo' to prevent attempted execution?
                        break;
                }
                if (propNameValue !== '') {
                    commandLineArgs.push(propNameValue);
                }
            }
            if (nogo) {
                commandLineArgs.push('-nogo');
            }
            // this is overriden if problemMatcher is set in the tasks.json file      
            let problemMatcher = "$saxon-xslt-js";
            const processExecution = new vscode.ProcessExecution(npxCommand, commandLineArgs.concat(xsltParametersCommand));
            let newTask = new vscode.Task(xsltTask, xsltTask.label, source, processExecution, problemMatcher);
            //let newTask = new vscode.Task(xsltTask, xsltTask.label, source, new vscode.ShellExecution(commandline), problemMatcher);
            return newTask;
        }
        else {
            return undefined;
        }
    }
}
exports.SaxonJsTaskProvider = SaxonJsTaskProvider;
SaxonJsTaskProvider.SaxonBuildScriptType = 'xslt-js';
//# sourceMappingURL=saxonJsTaskProvider.js.map