"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const xslLexer_1 = require("./xslLexer");
const xsltTokenDiagnostics_1 = require("./xsltTokenDiagnostics");
const globalsProvider_1 = require("./globalsProvider");
const path = require("path");
class DCPSymbolProvider {
    constructor(xsltConfiguration, collection) {
        this.xslLexer = new xslLexer_1.XslLexer(xsltConfiguration);
        this.xslLexer.provideCharLevelState = true;
        this.collection = collection;
        this.languageConfig = xsltConfiguration;
        this.docType = xsltConfiguration.docType;
    }
    provideDocumentSymbols(document, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = this.xslLexer.analyse(document.getText());
            let globals = this.xslLexer.globalInstructionData;
            function returnBadFileLinks(item) {
                return __awaiter(this, void 0, void 0, function* () {
                    const basePath = path.dirname(document.fileName);
                    const resolvedPath = path.resolve(basePath, item.name);
                    let fileExists = yield globalsProvider_1.GlobalsProvider.fileExists(resolvedPath);
                    if (fileExists) {
                        return undefined;
                    }
                    else {
                        return item;
                    }
                });
            }
            let fileChecks = [];
            globals.forEach((item) => {
                if (item.type === xslLexer_1.GlobalInstructionType.Import || item.type === xslLexer_1.GlobalInstructionType.Include) {
                    fileChecks.push(returnBadFileLinks(item));
                }
            });
            let errorFileRefs = yield Promise.all(fileChecks);
            return new Promise((resolve, reject) => {
                let symbols = [];
                let diagnostics = xsltTokenDiagnostics_1.XsltTokenDiagnostics.calculateDiagnostics(this.languageConfig, this.docType, document, allTokens, [], [], symbols);
                let importDiagnostics = [];
                errorFileRefs.forEach((fileRef) => {
                    if (fileRef) {
                        importDiagnostics.push(xsltTokenDiagnostics_1.XsltTokenDiagnostics.createImportDiagnostic(fileRef));
                    }
                });
                let allDiagnostics = importDiagnostics.concat(diagnostics);
                if (allDiagnostics.length > 0) {
                    this.collection.set(document.uri, allDiagnostics);
                }
                else {
                    this.collection.clear();
                }
                ;
                resolve(symbols);
            });
        });
    }
}
exports.DCPSymbolProvider = DCPSymbolProvider;
//# sourceMappingURL=dcpSymbolProvider.js.map