"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 *  Copyright (c) 2020 DeltaXML Ltd. and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the MIT license
 *  which accompanies this distribution.
 *
 *  Contributors:
 *  DeltaXML Ltd. - XPath/XSLT Lexer/Syntax Highlighter
 */
const vscode = require("vscode");
const xpLexer_1 = require("./xpLexer");
const xmlDocumentFormattingProvider_1 = require("./xmlDocumentFormattingProvider");
const saxonTaskProvider_1 = require("./saxonTaskProvider");
const saxonJsTaskProvider_1 = require("./saxonJsTaskProvider");
const languageConfigurations_1 = require("./languageConfigurations");
const xsltSymbolProvider_1 = require("./xsltSymbolProvider");
const xslLexer_1 = require("./xslLexer");
const documentChangeHandler_1 = require("./documentChangeHandler");
const xsltDefinitionProvider_1 = require("./xsltDefinitionProvider");
const documentLinkProvider_1 = require("./documentLinkProvider");
const fullDocumentLinkProvider_1 = require("./fullDocumentLinkProvider");
const dcpSymbolProvider_1 = require("./dcpSymbolProvider");
const xsltTokenDiagnostics_1 = require("./xsltTokenDiagnostics");
const vscode_1 = require("vscode");
const xsltHoverProvider_1 = require("./xsltHoverProvider");
const os = require("os");
const tokenModifiers = new Map();
const legend = (function () {
    const tokenTypesLegend = xslLexer_1.XslLexer.getTextmateTypeLegend();
    const tokenModifiersLegend = [
        'declaration', 'documentation', 'member', 'static', 'abstract', 'deprecated',
        'modification', 'async'
    ];
    tokenModifiersLegend.forEach((tokenModifier, index) => tokenModifiers.set(tokenModifier, index));
    return new vscode.SemanticTokensLegend(tokenTypesLegend, tokenModifiersLegend);
})();
function activate(context) {
    documentChangeHandler_1.DocumentChangeHandler.isWindowsOS = os.platform() === 'win32';
    const xsltDiagnosticsCollection = vscode.languages.createDiagnosticCollection('xslt');
    const xsltSymbolProvider = new xsltSymbolProvider_1.XsltSymbolProvider(languageConfigurations_1.XSLTConfiguration.configuration, xsltDiagnosticsCollection);
    const dcpDiagnosticsCollection = vscode.languages.createDiagnosticCollection('dcp');
    const dcpSymbolProvider = new dcpSymbolProvider_1.DCPSymbolProvider(languageConfigurations_1.DCPConfiguration.configuration, dcpDiagnosticsCollection);
    const dcpDefintiionProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(languageConfigurations_1.DCPConfiguration.configuration);
    const dcpLinkProvider = new fullDocumentLinkProvider_1.FullDocumentLinkProvider(languageConfigurations_1.DCPConfiguration.configuration);
    const schDiagnosticsCollection = vscode.languages.createDiagnosticCollection('sch');
    const schSymbolProvider = new xsltSymbolProvider_1.XsltSymbolProvider(languageConfigurations_1.SchConfiguration.configuration, schDiagnosticsCollection);
    const schDefintiionProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(languageConfigurations_1.SchConfiguration.configuration);
    const schLinkProvider = new fullDocumentLinkProvider_1.FullDocumentLinkProvider(languageConfigurations_1.SchConfiguration.configuration);
    const xmlDefinitionProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(languageConfigurations_1.XMLConfiguration.configuration);
    const xpathDefinitionProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(languageConfigurations_1.XPathConfiguration.configuration);
    const xsltDefintiionProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(languageConfigurations_1.XSLTConfiguration.configuration);
    const xsltLinkProvider = new documentLinkProvider_1.DocumentLinkProvider(languageConfigurations_1.XSLTLightConfiguration.configuration);
    const xmlDiagnosticsCollection = vscode.languages.createDiagnosticCollection('xml');
    const bpmnDiagnosticsCollection = vscode.languages.createDiagnosticCollection('bpmn');
    const xmlSymbolProvider = new xsltSymbolProvider_1.XsltSymbolProvider(languageConfigurations_1.XMLConfiguration.configuration, xmlDiagnosticsCollection);
    const bpmnSymbolProvider = new xsltSymbolProvider_1.XsltSymbolProvider(languageConfigurations_1.XMLConfiguration.configuration, bpmnDiagnosticsCollection);
    const docChangeHandler = new documentChangeHandler_1.DocumentChangeHandler();
    let activeEditor = vscode.window.activeTextEditor;
    if (activeEditor) {
        docChangeHandler.registerXMLEditor(activeEditor);
    }
    function showGotoXPathInputBox() {
        return __awaiter(this, void 0, void 0, function* () {
            let symbol;
            const xpath = xsltSymbolProvider_1.XsltSymbolProvider.getXPathFromSelection();
            const inboxValue = xpath ? xpath : '';
            const result = yield vscode_1.window.showInputBox({
                value: inboxValue,
                valueSelection: [0, inboxValue.length],
                placeHolder: '/element/element[2]/element[3]',
                validateInput: text => {
                    if (!text.startsWith('/')) {
                        return 'XPath should start with "/"';
                    }
                    else {
                        symbol = xsltSymbolProvider_1.XsltSymbolProvider.getSymbolFromXPathLocator(text, xsltSymbolProvider_1.XsltSymbolProvider.documentSymbols);
                        return symbol ? null : 'No matching elements';
                    }
                }
            });
            if (result) {
                xsltSymbolProvider_1.XsltSymbolProvider.selectTextWithSymbol(symbol);
                const foundSymbol = symbol !== undefined;
                const msg = foundSymbol ? `Matching element found: ${symbol === null || symbol === void 0 ? void 0 : symbol.name}` : 'No matching elements';
                vscode_1.window.showInformationMessage(msg);
            }
        });
    }
    function selectTextFromXPath(xpathText) {
        const symbol = xsltSymbolProvider_1.XsltSymbolProvider.getSymbolFromXPathLocator(xpathText, xsltSymbolProvider_1.XsltSymbolProvider.documentSymbols);
        if (symbol) {
            xsltSymbolProvider_1.XsltSymbolProvider.selectTextWithSymbol(symbol);
        }
    }
    let cachedSymbols = [];
    let cachedSymbolsDocUri = vscode.Uri.parse('file:/empty');
    function getSymbolFromXPath(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { xpath, uri } = args[0];
            const docUri = vscode.Uri.parse(uri);
            const useCachedSymbols = cachedSymbolsDocUri === docUri;
            const docs = vscode.workspace.textDocuments;
            const foundDoc = docs.find(doc => doc.uri.toString() === uri);
            if (!useCachedSymbols && foundDoc) {
                const sp = new xsltSymbolProvider_1.XsltSymbolProvider(languageConfigurations_1.XMLConfiguration.configuration, null);
                const newSymbols = yield sp.getDocumentSymbols(foundDoc);
                if (newSymbols) {
                    cachedSymbols = newSymbols;
                    cachedSymbolsDocUri = foundDoc.uri;
                }
            }
            const symbol = xsltSymbolProvider_1.XsltSymbolProvider.getSymbolFromXPathLocator(xpath, cachedSymbols);
            if (symbol) {
                return symbol;
            }
        });
    }
    context.subscriptions.push(vscode.window.onDidChangeActiveTextEditor(editor => {
        docChangeHandler.registerXMLEditor(editor);
    }));
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ language: 'xslt' }, xsltSymbolProvider));
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ language: 'dcp' }, dcpSymbolProvider));
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ language: 'sch' }, schSymbolProvider));
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ language: 'xml' }, xmlSymbolProvider));
    context.subscriptions.push(vscode.languages.registerDocumentSymbolProvider({ language: 'bpmn' }, bpmnSymbolProvider));
    context.subscriptions.push(vscode.languages.registerDefinitionProvider({ language: 'xslt' }, xsltDefintiionProvider));
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider({ language: 'xslt' }, xsltDefintiionProvider));
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider({ language: 'dcp' }, dcpDefintiionProvider));
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider({ language: 'sch' }, schDefintiionProvider));
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider({ language: 'xml' }, xmlDefinitionProvider));
    context.subscriptions.push(vscode.languages.registerCompletionItemProvider({ language: 'xpath' }, xpathDefinitionProvider));
    context.subscriptions.push(vscode.languages.registerDocumentLinkProvider({ language: 'xslt' }, xsltLinkProvider));
    context.subscriptions.push(vscode.languages.registerDocumentLinkProvider({ language: 'dcp' }, dcpLinkProvider));
    context.subscriptions.push(vscode.languages.registerDocumentLinkProvider({ language: 'sch' }, schLinkProvider));
    context.subscriptions.push(vscode.languages.registerHoverProvider({ language: 'xslt' }, new xsltHoverProvider_1.XSLTHoverProvider()));
    context.subscriptions.push(vscode.languages.registerHoverProvider({ language: 'xpath' }, new xsltHoverProvider_1.XSLTHoverProvider()));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.gotoXPath', () => showGotoXPathInputBox()));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.selectCurrentElement', () => xsltSymbolProvider_1.XsltSymbolProvider.selectXMLElement(xsltSymbolProvider_1.SelectionType.Current)));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.selectPrecedingElement', () => xsltSymbolProvider_1.XsltSymbolProvider.selectXMLElement(xsltSymbolProvider_1.SelectionType.Previous)));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.selectFollowingElement', () => xsltSymbolProvider_1.XsltSymbolProvider.selectXMLElement(xsltSymbolProvider_1.SelectionType.Next)));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.selectParentElement', () => xsltSymbolProvider_1.XsltSymbolProvider.selectXMLElement(xsltSymbolProvider_1.SelectionType.Parent)));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.selectXPath', (args) => selectTextFromXPath(args[0])));
    context.subscriptions.push(vscode.commands.registerCommand('xslt-xpath.symbolFromXPath', (...args) => getSymbolFromXPath(args)));
    // syntax highlighters
    context.subscriptions.push(vscode.languages.registerDocumentSemanticTokensProvider({ language: 'xslt' }, new XsltSemanticTokensProvider(languageConfigurations_1.XSLTConfiguration.configuration), legend));
    context.subscriptions.push(vscode.languages.registerDocumentSemanticTokensProvider({ language: 'dcp' }, new XsltSemanticTokensProvider(languageConfigurations_1.DCPConfiguration.configuration), legend));
    context.subscriptions.push(vscode.languages.registerDocumentSemanticTokensProvider({ language: 'sch' }, new XsltSemanticTokensProvider(languageConfigurations_1.SchConfiguration.configuration), legend));
    context.subscriptions.push(vscode.languages.registerDocumentSemanticTokensProvider({ language: 'bpmn' }, new XsltSemanticTokensProvider(languageConfigurations_1.XMLConfiguration.configuration), legend));
    const xpathDiagnosticsCollection = vscode.languages.createDiagnosticCollection('xpath');
    context.subscriptions.push(vscode.languages.registerDocumentSemanticTokensProvider({ language: 'xpath' }, new XPathSemanticTokensProvider(xpathDiagnosticsCollection), legend));
    // formatter
    let xsltFormatter = new xmlDocumentFormattingProvider_1.XMLDocumentFormattingProvider(languageConfigurations_1.XSLTConfiguration.configuration);
    let xpathFormatter = new xmlDocumentFormattingProvider_1.XMLDocumentFormattingProvider(languageConfigurations_1.XPathConfiguration.configuration);
    let xmlFormatter = new xmlDocumentFormattingProvider_1.XMLDocumentFormattingProvider(languageConfigurations_1.XMLConfiguration.configuration);
    let dcpFormatter = new xmlDocumentFormattingProvider_1.XMLDocumentFormattingProvider(languageConfigurations_1.DCPConfiguration.configuration);
    let schFormatter = new xmlDocumentFormattingProvider_1.XMLDocumentFormattingProvider(languageConfigurations_1.SchConfiguration.configuration);
    context.subscriptions.push(vscode.languages.registerDocumentFormattingEditProvider('xslt', xsltFormatter));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider('xslt', xsltFormatter));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider('xslt', xsltFormatter, '\n', '/'));
    context.subscriptions.push(vscode.languages.registerDocumentFormattingEditProvider('xpath', xpathFormatter));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider('xpath', xpathFormatter));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider('xpath', xpathFormatter, '\n', '/'));
    context.subscriptions.push(vscode.languages.registerDocumentFormattingEditProvider(['xml', 'bpmn'], xmlFormatter));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider(['xml', 'bpmn'], xmlFormatter));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider(['xml', 'bpmn'], xmlFormatter, '\n', '/'));
    context.subscriptions.push(vscode.languages.registerDocumentFormattingEditProvider('dcp', dcpFormatter));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider('dcp', dcpFormatter));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider('dcp', dcpFormatter, '\n', '/'));
    context.subscriptions.push(vscode.languages.registerDocumentFormattingEditProvider('sch', schFormatter));
    context.subscriptions.push(vscode.languages.registerDocumentRangeFormattingEditProvider('sch', schFormatter));
    context.subscriptions.push(vscode.languages.registerOnTypeFormattingEditProvider('sch', schFormatter, '\n', '/'));
    let workspaceRoot = vscode.workspace.rootPath;
    if (!workspaceRoot) {
        return;
    }
    let xsltTaskEnabled = vscode.workspace.getConfiguration('XSLT.tasks.java').get('enabled');
    if (xsltTaskEnabled) {
        let xsltTaskProvider = vscode.tasks.registerTaskProvider(saxonTaskProvider_1.SaxonTaskProvider.SaxonBuildScriptType, new saxonTaskProvider_1.SaxonTaskProvider(workspaceRoot));
        context.subscriptions.push(xsltTaskProvider);
    }
    let xsltJsTaskEnabled = vscode.workspace.getConfiguration('XSLT.tasks.js').get('enabled');
    if (xsltJsTaskEnabled) {
        let xsltjsTaskProvider = vscode.tasks.registerTaskProvider(saxonJsTaskProvider_1.SaxonJsTaskProvider.SaxonBuildScriptType, new saxonJsTaskProvider_1.SaxonJsTaskProvider(workspaceRoot));
        context.subscriptions.push(xsltjsTaskProvider);
    }
}
exports.activate = activate;
class XPathSemanticTokensProvider {
    constructor(collection) {
        this.xpLexer = new xpLexer_1.XPathLexer();
        this.collection = collection;
    }
    provideDocumentSemanticTokens(document, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const lexPosition = { line: 0, startCharacter: 0, documentOffset: 0 };
            this.xpLexer.documentTokens = [];
            const allTokens = this.xpLexer.analyse(document.getText(), xpLexer_1.ExitCondition.None, lexPosition);
            setTimeout(() => this.reportProblems(allTokens, document), 0);
            const builder = new vscode.SemanticTokensBuilder();
            allTokens.forEach((token) => {
                builder.push(token.line, token.startCharacter, token.length, token.tokenType, 0);
            });
            return builder.build();
        });
    }
    reportProblems(allTokens, document) {
        let diagnostics = xsltTokenDiagnostics_1.XsltTokenDiagnostics.calculateDiagnostics(languageConfigurations_1.XPathConfiguration.configuration, xslLexer_1.DocumentTypes.XPath, document, allTokens, documentChangeHandler_1.DocumentChangeHandler.lastXMLDocumentGlobalData, [], []);
        if (diagnostics.length > 0) {
            this.collection.set(document.uri, diagnostics);
        }
        else {
            this.collection.clear();
        }
        ;
    }
}
class XsltSemanticTokensProvider {
    constructor(languageConfig) {
        this.xslLexer = new xslLexer_1.XslLexer(languageConfig);
        this.xslLexer.provideCharLevelState = true;
    }
    provideDocumentSemanticTokens(document, token) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('provideDocumentSemanticTokens');
            const allTokens = this.xslLexer.analyse(document.getText());
            const builder = new vscode.SemanticTokensBuilder();
            allTokens.forEach((token) => {
                builder.push(token.line, token.startCharacter, token.length, token.tokenType, 0);
            });
            return builder.build();
        });
    }
}
exports.XsltSemanticTokensProvider = XsltSemanticTokensProvider;
//# sourceMappingURL=extension.js.map