"use strict";
/**
 *  Copyright (c) 2020 DeltaXML Ltd. and others.
 *  All rights reserved. This program and the accompanying materials
 *  are made available under the terms of the MIT license
 *  which accompanies this distribution.
 *
 *  Contributors:
 *  DeltaXML Ltd. - XPath/XSLT Lexer/Syntax Highlighter
 */
Object.defineProperty(exports, "__esModule", { value: true });
const xslLexer_1 = require("./xslLexer");
const xpLexer_1 = require("./xpLexer");
class XslLexerLight extends xslLexer_1.XslLexer {
    analyseLight(xsl, xmlnsPrefixesOnly) {
        this.globalInstructionData = [];
        this.globalModeData = [];
        let currentState = xslLexer_1.XMLCharState.init;
        let currentChar = '';
        let tokenChars = [];
        let attName = '';
        let xpLexer = new xpLexer_1.XPathLexer();
        xpLexer.documentText = xsl;
        let xslLength = xsl.length - 1;
        let storeToken = false;
        let isNativeElement = false;
        let tagGlobalInstructionType = xslLexer_1.GlobalInstructionType.Unknown;
        let tagInstructionNameAdded = false;
        let tagMatchToken = null;
        let contextGlobalInstructionType = xslLexer_1.GlobalInstructionType.Unknown;
        let isGlobalInstructionName = false;
        let isGlobalInstructionMode = false;
        let isGlobalParameterName = false;
        let isGlobalUsePackageVersion = false;
        let isGlobalInstructionMatch = false;
        let xmlElementStack = 0;
        let lCharCount = -1;
        let lineNumber = 0;
        let lineNumberChar = -1;
        let collectParamName = false;
        while (lCharCount < xslLength + 1) {
            lCharCount++;
            lineNumberChar++;
            let nextState = xslLexer_1.XMLCharState.init;
            let nextChar = xsl.charAt(lCharCount);
            if (currentChar) {
                let isCurrentCharNewLIne = currentChar === '\n';
                if (isCurrentCharNewLIne) {
                    lineNumberChar = 0;
                    lineNumber++;
                }
                nextState = this.calcNewState(isCurrentCharNewLIne, currentChar, nextChar, currentState);
                if (nextState === currentState) {
                    if (isCurrentCharNewLIne) {
                        // do nothing
                    }
                    else if (storeToken) {
                        tokenChars.push(currentChar);
                    }
                }
                else {
                    switch (nextState) {
                        case xslLexer_1.XMLCharState.lSt:
                        case xslLexer_1.XMLCharState.lCtName:
                            break;
                        case xslLexer_1.XMLCharState.lEn:
                            if (xmlElementStack === 1 || xmlElementStack === 2 && tokenChars.length < 5) {
                                tokenChars.push(currentChar);
                                storeToken = true;
                            }
                            else {
                                storeToken = false;
                            }
                            break;
                        case xslLexer_1.XMLCharState.rStNoAtt:
                            xmlElementStack++;
                        case xslLexer_1.XMLCharState.lsElementNameWs:
                        case xslLexer_1.XMLCharState.rSelfCtNoAtt:
                        case xslLexer_1.XMLCharState.rCt:
                            let isCloseTag = nextState === xslLexer_1.XMLCharState.rCt;
                            let isRootChildStartTag = !isCloseTag && xmlElementStack === 1;
                            let elementProperties = this.getElementProperties(tokenChars, isRootChildStartTag);
                            isNativeElement = elementProperties.isNative;
                            tagGlobalInstructionType = elementProperties.instructionType;
                            tagInstructionNameAdded = false;
                            tagMatchToken = null;
                            collectParamName = false;
                            if (xmlElementStack === 1) {
                                contextGlobalInstructionType = tagGlobalInstructionType;
                            }
                            else if (xmlElementStack === 2
                                && (contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Function || contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Template)
                                && isNativeElement && elementProperties.nativeName === 'param') {
                                collectParamName = true;
                            }
                            if (isCloseTag) {
                                if (xmlElementStack > 0) {
                                    xmlElementStack--;
                                }
                            }
                            storeToken = false;
                            tokenChars = [];
                            break;
                        case xslLexer_1.XMLCharState.wsAfterAttName:
                        case xslLexer_1.XMLCharState.syntaxError:
                            storeToken = false;
                            break;
                        case xslLexer_1.XMLCharState.lAn:
                            if ((xmlElementStack === 1 && tagGlobalInstructionType !== xslLexer_1.GlobalInstructionType.Unknown) ||
                                xmlElementStack === 2 && contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Function || contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Template || contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.UsePackage) {
                                tokenChars.push(currentChar);
                                storeToken = true;
                            }
                            else if (xmlElementStack < 3 && xmlnsPrefixesOnly) {
                                tokenChars.push(currentChar);
                                storeToken = true;
                            }
                            break;
                        case xslLexer_1.XMLCharState.lStEq:
                            attName = tokenChars.join('');
                            isGlobalInstructionName = false;
                            isGlobalInstructionMode = false;
                            isGlobalParameterName = false;
                            isGlobalUsePackageVersion = false;
                            isGlobalInstructionMatch = false;
                            if ((tagGlobalInstructionType === xslLexer_1.GlobalInstructionType.Include || tagGlobalInstructionType === xslLexer_1.GlobalInstructionType.Import)
                                && attName === 'href') {
                                isGlobalInstructionName = true;
                            }
                            else if (tagGlobalInstructionType !== xslLexer_1.GlobalInstructionType.Unknown && attName === 'name') {
                                isGlobalInstructionName = true;
                            }
                            else if (tagGlobalInstructionType == xslLexer_1.GlobalInstructionType.Template && attName === 'mode' || tagGlobalInstructionType === xslLexer_1.GlobalInstructionType.Mode && attName === 'name') {
                                isGlobalInstructionMode = true;
                            }
                            else if (tagGlobalInstructionType == xslLexer_1.GlobalInstructionType.Template && attName === 'match') {
                                isGlobalInstructionMatch = true;
                            }
                            else if (collectParamName && attName === 'name') {
                                isGlobalParameterName = true;
                            }
                            else if (contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.UsePackage && attName === 'package-version') {
                                isGlobalUsePackageVersion = true;
                            }
                            else if (xmlnsPrefixesOnly && attName.startsWith('xmlns:')) {
                                let xmlnsTkn = {
                                    line: lineNumber,
                                    length: attName.length,
                                    startCharacter: lineNumberChar,
                                    value: attName,
                                    tokenType: xslLexer_1.XSLTokenLevelState.attributeName
                                };
                                const gData = {
                                    idNumber: 0,
                                    name: attName.substring(6),
                                    token: xmlnsTkn,
                                    type: xslLexer_1.GlobalInstructionType.RootXMLNS
                                };
                                this.globalInstructionData.push(gData);
                            }
                            tokenChars = [];
                            storeToken = false;
                            break;
                        case xslLexer_1.XMLCharState.rSt:
                            if (tagGlobalInstructionType === xslLexer_1.GlobalInstructionType.Template && !tagInstructionNameAdded && tagMatchToken) {
                                this.globalInstructionData.push({ type: xslLexer_1.GlobalInstructionType.TemplateMatch, name: `${tagMatchToken.value}#${this.globalInstructionData.length}`, token: tagMatchToken, idNumber: 0 });
                            }
                            xmlElementStack++;
                            storeToken = false;
                            tokenChars = [];
                            break;
                        case xslLexer_1.XMLCharState.rSq:
                        case xslLexer_1.XMLCharState.rDq:
                        case xslLexer_1.XMLCharState.escDqAvt:
                        case xslLexer_1.XMLCharState.escSqAvt:
                            if (isGlobalInstructionName || isGlobalInstructionMode) {
                                let attValue = tokenChars.join('');
                                let tkn = {
                                    line: lineNumber,
                                    length: attValue.length - 2,
                                    startCharacter: lineNumberChar - (attValue.length + 1),
                                    value: attValue,
                                    tokenType: xslLexer_1.XSLTokenLevelState.attributeValue
                                };
                                let globalType = isGlobalInstructionMode ? xslLexer_1.GlobalInstructionType.Mode : tagGlobalInstructionType;
                                let targetGlobal = isGlobalInstructionMode ? this.globalModeData : this.globalInstructionData;
                                if (isGlobalInstructionMode) {
                                    targetGlobal = this.globalModeData;
                                }
                                else {
                                    targetGlobal = this.globalInstructionData;
                                    tagInstructionNameAdded = true;
                                }
                                targetGlobal.push({ type: globalType, name: attValue, token: tkn, idNumber: 0 });
                                isGlobalInstructionName = false;
                                tagGlobalInstructionType = xslLexer_1.GlobalInstructionType.Unknown;
                            }
                            else if (isGlobalParameterName || isGlobalUsePackageVersion) {
                                let attValue = tokenChars.join('');
                                if (this.globalInstructionData.length > 0) {
                                    let gd = this.globalInstructionData[this.globalInstructionData.length - 1];
                                    if (isGlobalUsePackageVersion) {
                                        gd['version'] = attValue;
                                    }
                                    else {
                                        if (gd.memberNames) {
                                            gd.memberNames.push(attValue);
                                        }
                                        else {
                                            gd['memberNames'] = [attValue];
                                        }
                                        gd.idNumber++;
                                    }
                                }
                            }
                            else if (isGlobalInstructionMatch) {
                                let attValue = tokenChars.join('');
                                let tkn = {
                                    line: lineNumber,
                                    length: attValue.length,
                                    startCharacter: lineNumberChar - attValue.length,
                                    value: attValue,
                                    tokenType: xslLexer_1.XSLTokenLevelState.attributeValue
                                };
                                tagMatchToken = tkn;
                                tagMatchToken.value = attValue;
                            }
                            tokenChars = [];
                            storeToken = false;
                            break;
                        case xslLexer_1.XMLCharState.lSq:
                        case xslLexer_1.XMLCharState.lDq:
                            if (contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Function || contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.Template || contextGlobalInstructionType === xslLexer_1.GlobalInstructionType.UsePackage
                                || isGlobalInstructionName || isGlobalInstructionMode) {
                                storeToken = true;
                            }
                            break;
                        case xslLexer_1.XMLCharState.sqAvt:
                        case xslLexer_1.XMLCharState.dqAvt:
                            nextState = nextState === xslLexer_1.XMLCharState.sqAvt ? xslLexer_1.XMLCharState.lSq : xslLexer_1.XMLCharState.lDq;
                            break;
                        case xslLexer_1.XMLCharState.tvt:
                        case xslLexer_1.XMLCharState.tvtCdata:
                            if (nextState === xslLexer_1.XMLCharState.tvtCdata) {
                                nextState = xslLexer_1.XMLCharState.awaitingRcdata;
                            }
                            break;
                        case xslLexer_1.XMLCharState.rEntity:
                            switch (this.entityContext) {
                                case xslLexer_1.EntityPosition.text:
                                    nextState = xslLexer_1.XMLCharState.init;
                                    break;
                                case xslLexer_1.EntityPosition.attrSq:
                                    nextState = xslLexer_1.XMLCharState.lSq;
                                    break;
                                case xslLexer_1.EntityPosition.attrDq:
                                    nextState = xslLexer_1.XMLCharState.lDq;
                                    break;
                            }
                            break;
                    }
                } // else ends
                currentState = nextState;
            }
            currentChar = nextChar;
        }
        return this.globalInstructionData.concat(this.globalModeData);
    }
}
exports.XslLexerLight = XslLexerLight;
//# sourceMappingURL=xslLexerLight.js.map