"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const xslLexer_1 = require("./xslLexer");
const globalsProvider_1 = require("./globalsProvider");
const xsltTokenDefintions_1 = require("./xsltTokenDefintions");
const xsltTokenCompletions_1 = require("./xsltTokenCompletions");
const xsltSymbolProvider_1 = require("./xsltSymbolProvider");
const xpLexer_1 = require("./xpLexer");
class XsltDefinitionProvider {
    constructor(xsltConfiguration) {
        this.xpLexer = null;
        this.gp = new globalsProvider_1.GlobalsProvider();
        this.languageConfig = xsltConfiguration;
        this.xslLexer = new xslLexer_1.XslLexer(xsltConfiguration);
        this.xslLexer.provideCharLevelState = true;
        this.docType = xsltConfiguration.docType;
        this.schemaData = xsltConfiguration.schemaData;
    }
    getXPLexer() {
        if (this.xpLexer === null) {
            this.xpLexer = new xpLexer_1.XPathLexer();
        }
        return this.xpLexer;
    }
    provideDefinition(document, position, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const lexPosition = { line: 0, startCharacter: 0, documentOffset: 0 };
            let allTokens = this.docType === xslLexer_1.DocumentTypes.XPath ?
                this.getXPLexer().analyse(document.getText(), xpLexer_1.ExitCondition.None, lexPosition) :
                this.xslLexer.analyse(document.getText());
            const globalInstructionData = this.xslLexer.globalInstructionData;
            const xsltPackages = vscode.workspace.getConfiguration('XSLT.resources').get('xsltPackages');
            // Import/include XSLT - ensuring no duplicates
            let importedG = { data: globalInstructionData, href: document.fileName, error: false };
            let importedGlobals1 = [importedG];
            let accumulatedHrefs = [importedG.href];
            let globalsSummary0 = { globals: importedGlobals1, hrefs: accumulatedHrefs };
            const maxImportLevel = 20;
            let processNestedGlobals = () => __awaiter(this, void 0, void 0, function* () {
                let level = 0;
                while (globalsSummary0.hrefs.length > 0 && level < maxImportLevel) {
                    globalsSummary0 = yield this.processImportedGlobals(xsltPackages, globalsSummary0.globals, accumulatedHrefs, level === 0);
                    level++;
                }
            });
            yield processNestedGlobals();
            return new Promise((resolve, reject) => {
                let location = undefined;
                let allImportedGlobals = [];
                globalsSummary0.globals.forEach((globals) => {
                    if (globals.error) {
                        // ignore 
                    }
                    else {
                        globals.data.forEach((global) => {
                            global['href'] = globals.href;
                            allImportedGlobals.push(global);
                        });
                    }
                });
                let isXSLT = this.docType === xslLexer_1.DocumentTypes.XSLT;
                location = xsltTokenDefintions_1.XsltTokenDefinitions.findDefinition(isXSLT, document, allTokens, globalInstructionData, allImportedGlobals, position);
                resolve(location);
            });
        });
    }
    provideCompletionItems(document, position, token, context) {
        return __awaiter(this, void 0, void 0, function* () {
            const keepNameTests = true;
            const lexPosition = { line: 0, startCharacter: 0, documentOffset: 0 };
            let allTokens = this.docType === xslLexer_1.DocumentTypes.XPath ?
                this.getXPLexer().analyse(document.getText(), xpLexer_1.ExitCondition.None, lexPosition) :
                this.xslLexer.analyse(document.getText(), keepNameTests);
            const globalInstructionData = this.xslLexer.globalInstructionData;
            const xsltPackages = vscode.workspace.getConfiguration('XSLT.resources').get('xsltPackages');
            // Import/include XSLT - ensuring no duplicates
            let importedG = { data: globalInstructionData, href: document.fileName, error: false };
            let importedGlobals1 = [importedG];
            let accumulatedHrefs = [importedG.href];
            let globalsSummary0 = { globals: importedGlobals1, hrefs: accumulatedHrefs };
            const maxImportLevel = 20;
            let processNestedGlobals = () => __awaiter(this, void 0, void 0, function* () {
                let level = 0;
                while (globalsSummary0.hrefs.length > 0 && level < maxImportLevel) {
                    globalsSummary0 = yield this.processImportedGlobals(xsltPackages, globalsSummary0.globals, accumulatedHrefs, level === 0);
                    level++;
                }
            });
            yield processNestedGlobals();
            return new Promise((resolve, reject) => {
                let allImportedGlobals = [];
                globalsSummary0.globals.forEach((globals) => {
                    if (globals.error) {
                        // ignore 
                    }
                    else {
                        globals.data.forEach((global) => {
                            global['href'] = globals.href;
                            allImportedGlobals.push(global);
                        });
                    }
                });
                let attNames = this.xslLexer.attributeNameTests ? this.xslLexer.attributeNameTests : [];
                let nodeNames = this.xslLexer.elementNameTests ? this.xslLexer.elementNameTests : [];
                let xslVariable = ['xsl:variable', 'xsl:param'];
                let completions;
                completions = xsltTokenCompletions_1.XsltTokenCompletions.getCompletions(this.languageConfig, xslVariable, attNames, nodeNames, document, allTokens, globalInstructionData, allImportedGlobals, position);
                resolve(completions);
            });
        });
    }
    processImportedGlobals(xsltPackages, importedGlobals1, level1Hrefs, topLevel) {
        return __awaiter(this, void 0, void 0, function* () {
            let level2Globals = [];
            let level2Hrefs = this.accumulateImportHrefs(xsltPackages, importedGlobals1, level1Hrefs);
            let newGlobals = [];
            level2Hrefs.forEach((href) => {
                level2Globals.push(this.fetchImportedGlobals([href]));
            });
            let importedGlobals2Array = yield Promise.all(level2Globals);
            importedGlobals2Array.forEach((importedGlobals2) => {
                if (topLevel) {
                    newGlobals = newGlobals.concat(importedGlobals2);
                }
                else {
                    importedGlobals1 = importedGlobals1.concat(importedGlobals2);
                }
            });
            if (topLevel) {
                return { globals: newGlobals, hrefs: level2Hrefs };
            }
            else {
                return { globals: importedGlobals1, hrefs: level2Hrefs };
            }
        });
    }
    accumulateImportHrefs(xsltPackages, importedGlobals, existingHrefs) {
        let result = [];
        const rootPath = vscode.workspace.rootPath;
        importedGlobals.forEach((importedG) => {
            importedG.data.forEach((data) => {
                if (data.type === xslLexer_1.GlobalInstructionType.Import || data.type === xslLexer_1.GlobalInstructionType.Include) {
                    let resolvedName = xsltSymbolProvider_1.XsltSymbolProvider.resolvePath(data.name, importedG.href);
                    if (existingHrefs.indexOf(resolvedName) < 0) {
                        existingHrefs.push(resolvedName);
                        result.push(resolvedName);
                    }
                }
                else if (rootPath && data.type === xslLexer_1.GlobalInstructionType.UsePackage) {
                    let packageLookup = xsltPackages.find((pkg) => {
                        return pkg.name === data.name;
                    });
                    if (packageLookup) {
                        let resolvedName = xsltSymbolProvider_1.XsltSymbolProvider.resolvePathInSettings(packageLookup.path, rootPath);
                        if (existingHrefs.indexOf(resolvedName) < 0) {
                            existingHrefs.push(resolvedName);
                            result.push(resolvedName);
                        }
                    }
                }
            });
        });
        return result;
    }
    fetchImportedGlobals(inputHrefs) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = [];
            //let inputHrefs: string[] = this.accumulateImportHrefs(globalInstructionData, existingHrefs, docHref);
            let lastIndex = inputHrefs.length - 1;
            if (lastIndex < 0) {
                return result;
            }
            else {
                return new Promise((resolve, reject) => {
                    inputHrefs.forEach((href, index) => {
                        this.gp.provideGlobals(href).then((globals) => {
                            result.push({ href: href, data: globals.data, error: globals.error });
                            if (index === lastIndex) {
                                resolve(result);
                            }
                        });
                    });
                });
            }
        });
    }
}
exports.XsltDefinitionProvider = XsltDefinitionProvider;
//# sourceMappingURL=xsltDefinitionProvider.js.map