"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const functionData_1 = require("./functionData");
const languageConfigurations_1 = require("./languageConfigurations");
const schemaQuery_1 = require("./schemaQuery");
const xpLexer_1 = require("./xpLexer");
const xslLexer_1 = require("./xslLexer");
const xsltDefinitionProvider_1 = require("./xsltDefinitionProvider");
const xsltTokenDefintions_1 = require("./xsltTokenDefintions");
const xsltTokenDiagnostics_1 = require("./xsltTokenDiagnostics");
const url = require("url");
class XSLTReferenceProvider {
    constructor() {
        this.xslLexer = new xslLexer_1.XslLexer(languageConfigurations_1.XSLTConfiguration.configuration);
    }
    provideReferences(document, position, context, token) {
        return __awaiter(this, void 0, void 0, function* () {
            const lexPosition = { line: 0, startCharacter: 0, documentOffset: 0 };
            const langConfig = languageConfigurations_1.XSLTConfiguration.configuration;
            // TODO: first check if position is on a definition already
            const dProvider = new xsltDefinitionProvider_1.XsltDefinitionProvider(langConfig);
            const definition = yield dProvider.seekDefinition(document, position, token);
            let locations = [];
            if (definition) {
                const { instruction, extractedImportData: eid } = definition;
                if (instruction && eid) {
                    // find all references to this instruction
                    let refTokens = XSLTReferenceProvider.calculateReferences(instruction, langConfig, langConfig.docType, document, eid.allTokens, eid.globalInstructionData, eid.allImportedGlobals);
                    const refLocations = refTokens.map(token => xsltTokenDefintions_1.XsltTokenDefinitions.createLocationFromToken(token, document));
                    locations = refLocations;
                    locations.push(definition);
                    for (let index = 0; index < eid.accumulatedHrefs.length; index++) {
                        try {
                            const pathForUri = url.pathToFileURL(eid.accumulatedHrefs[index]).toString();
                            const docUri = vscode.Uri.parse(pathForUri);
                            let hrefDoc = yield vscode.workspace.openTextDocument(docUri);
                            const hrefAllTokens = this.xslLexer.analyse(hrefDoc.getText());
                            const hrefGlobalInstructionData = this.xslLexer.globalInstructionData;
                            let refDocTokens = XSLTReferenceProvider.calculateReferences(instruction, langConfig, langConfig.docType, hrefDoc, hrefAllTokens, hrefGlobalInstructionData, eid.allImportedGlobals);
                            const refLocations = refDocTokens.map(token => xsltTokenDefintions_1.XsltTokenDefinitions.createLocationFromToken(token, hrefDoc));
                            locations = locations.concat(refLocations);
                        }
                        catch (error) {
                            console.error(error);
                        }
                    }
                }
            }
            else {
            }
            return new Promise(resolve => {
                resolve(locations);
            });
        });
    }
}
exports.XSLTReferenceProvider = XSLTReferenceProvider;
XSLTReferenceProvider.calculateReferences = (seekInstruction, languageConfig, docType, document, allTokens, globalInstructionData, importedInstructionData) => {
    let lineNumber = -1;
    let xslVariable = languageConfig.variableElementNames;
    let inScopeVariablesList = [];
    let xpathVariableCurrentlyBeingDefined;
    let elementStack = [];
    let inScopeXPathVariablesList = [];
    let anonymousFunctionParamList = [];
    let xpathStack = [];
    let tagType = xsltTokenDiagnostics_1.TagType.NonStart;
    let attType = xsltTokenDiagnostics_1.AttributeType.None;
    let tagElementName = '';
    let tagElementChildren = [];
    let startTagToken = null;
    let preXPathVariable = false;
    let anonymousFunctionParams = false;
    let variableData = null;
    let xsltVariableDeclarations = [];
    let unresolvedXsltVariableReferences = [];
    let prevToken = null;
    let includeOrImport = false;
    let problemTokens = [];
    let referenceTokens = [];
    let tagIdentifierName = '';
    let lastTokenIndex = allTokens.length - 1;
    let tagAttributeNames = [];
    let tagAttributeSymbols = [];
    let tagXmlnsNames = [];
    let rootXmlnsBindings = [];
    let inheritedPrefixes = [];
    let globalVariableData = [];
    let importedGlobalVarNames = [];
    let importedGlobalFnNames = [];
    let incrementFunctionArity = false;
    let onRootStartTag = true;
    let rootXmlnsName = null;
    let xsltPrefixesToURIs = new Map();
    let namedTemplates = new Map();
    let globalModes = ['#current', '#default'];
    let globalKeys = [];
    let globalAccumulatorNames = [];
    let globalAttributeSetNames = [];
    let tagExcludeResultPrefixes = null;
    let ifThenStack = [];
    let currentXSLTIterateParams = [];
    let schemaQuery = languageConfig.schemaData ? new schemaQuery_1.SchemaQuery(languageConfig.schemaData) : undefined;
    let xsltSchemaQuery;
    const isSchematron = docType === xslLexer_1.DocumentTypes.SCH;
    let pendingTemplateParamErrors = [];
    if (isSchematron && languageConfigurations_1.XSLTConfiguration.configuration.schemaData) {
        xsltSchemaQuery = new schemaQuery_1.SchemaQuery(languageConfigurations_1.XSLTConfiguration.configuration.schemaData);
    }
    globalInstructionData.forEach((instruction) => {
        switch (instruction.type) {
            case xslLexer_1.GlobalInstructionType.Variable:
            case xslLexer_1.GlobalInstructionType.Parameter:
                globalVariableData.push({ token: instruction.token, name: instruction.name });
                xsltVariableDeclarations.push(instruction.token);
                break;
            case xslLexer_1.GlobalInstructionType.Function:
                let functionNameWithArity = instruction.name + '#' + instruction.idNumber;
                importedGlobalFnNames.push(functionNameWithArity);
                break;
            case xslLexer_1.GlobalInstructionType.Template:
                if (namedTemplates.get(instruction.name)) {
                    instruction.token['error'] = xpLexer_1.ErrorType.DuplicateTemplateName;
                    instruction.token.value = instruction.name;
                    problemTokens.push(instruction.token);
                }
                else {
                    let members = instruction.memberNames ? instruction.memberNames : [];
                    namedTemplates.set(instruction.name, members);
                }
                break;
            case xslLexer_1.GlobalInstructionType.Mode:
                let modes = instruction.name.split(/\s+/);
                globalModes = globalModes.concat(modes);
                break;
            case xslLexer_1.GlobalInstructionType.Key:
                globalKeys.push(instruction.name);
                break;
            case xslLexer_1.GlobalInstructionType.Accumulator:
                if (globalAccumulatorNames.indexOf(instruction.name) < 0) {
                    globalAccumulatorNames.push(instruction.name);
                }
                else {
                    instruction.token['error'] = xpLexer_1.ErrorType.DuplicateAccumulatorName;
                    instruction.token.value = instruction.name;
                    problemTokens.push(instruction.token);
                }
                break;
            case xslLexer_1.GlobalInstructionType.AttributeSet:
                globalAttributeSetNames.push(instruction.name);
                break;
            case xslLexer_1.GlobalInstructionType.RootXMLNS:
                if (docType === xslLexer_1.DocumentTypes.XPath) {
                    inheritedPrefixes.push(instruction.name);
                }
                break;
        }
    });
    importedInstructionData.forEach((instruction) => {
        switch (instruction.type) {
            case xslLexer_1.GlobalInstructionType.Variable:
            case xslLexer_1.GlobalInstructionType.Parameter:
                importedGlobalVarNames.push(instruction.name);
                break;
            case xslLexer_1.GlobalInstructionType.Function:
                let functionNameWithArity = instruction.name + '#' + instruction.idNumber;
                importedGlobalFnNames.push(functionNameWithArity);
                break;
            case xslLexer_1.GlobalInstructionType.Template:
                let members = instruction.memberNames ? instruction.memberNames : [];
                namedTemplates.set(instruction.name, members);
                break;
            case xslLexer_1.GlobalInstructionType.Mode:
                let modes = instruction.name.split(/\s+/);
                globalModes = globalModes.concat(modes);
                break;
            case xslLexer_1.GlobalInstructionType.Key:
                globalKeys.push(instruction.name);
                break;
            case xslLexer_1.GlobalInstructionType.Accumulator:
                globalAccumulatorNames.push(instruction.name);
                break;
            case xslLexer_1.GlobalInstructionType.AttributeSet:
                globalAttributeSetNames.push(instruction.name);
                break;
        }
    });
    if (docType === xslLexer_1.DocumentTypes.XPath) {
        xsltPrefixesToURIs.set('array', functionData_1.XSLTnamespaces.Array);
        xsltPrefixesToURIs.set('map', functionData_1.XSLTnamespaces.Map);
        xsltPrefixesToURIs.set('math', functionData_1.XSLTnamespaces.Map);
        xsltPrefixesToURIs.set('xs', functionData_1.XSLTnamespaces.XMLSchema);
        xsltPrefixesToURIs.set('fn', functionData_1.XSLTnamespaces.XPath);
        xsltPrefixesToURIs.set('xsl', functionData_1.XSLTnamespaces.XSLT);
        xsltPrefixesToURIs.set('ixsl', functionData_1.XSLTnamespaces.IXSL);
        inheritedPrefixes = inheritedPrefixes.concat(['array', 'map', 'math', 'xs', 'fn', 'xsl', 'ixsl']);
    }
    allTokens.forEach((token, index) => {
        var _a, _b;
        lineNumber = token.line;
        let isXMLToken = token.tokenType >= xsltTokenDiagnostics_1.XsltTokenDiagnostics.xsltStartTokenNumber;
        if (isXMLToken) {
            if (ifThenStack.length > 0) {
                let ifToken = ifThenStack[0];
                ifToken['error'] = xpLexer_1.ErrorType.BracketNesting;
                problemTokens.push(ifToken);
                ifThenStack = [];
            }
            inScopeXPathVariablesList = [];
            xpathVariableCurrentlyBeingDefined = false;
            if (xpathStack.length > 0) {
                // report last issue with nesting in each xpath:
                let errorToken;
                for (let index = xpathStack.length - 1; index > -1; index--) {
                    const trailingToken = xpathStack[index].token;
                    const tv = trailingToken.value;
                    const allowedToken = (tv === 'return' || tv === 'else' || tv === 'satisfies');
                    if (!allowedToken) {
                        errorToken = trailingToken;
                        break;
                    }
                }
                if (errorToken) {
                    errorToken['error'] = xpLexer_1.ErrorType.BracketNesting;
                    problemTokens.push(errorToken);
                }
            }
            xpathStack = [];
            preXPathVariable = false;
            let xmlCharType = token.charType;
            let xmlTokenType = (token.tokenType - xsltTokenDiagnostics_1.XsltTokenDiagnostics.xsltStartTokenNumber);
            switch (xmlTokenType) {
                case xslLexer_1.XSLTokenLevelState.xmlText:
                    if (elementStack.length === 0 && token.startCharacter > -1) {
                        const tValue = xsltTokenDiagnostics_1.XsltTokenDiagnostics.getTextForToken(lineNumber, token, document);
                        if (tValue.trim().length !== 0) {
                            token['error'] = xpLexer_1.ErrorType.ParentLessText;
                            token['value'] = tValue;
                            problemTokens.push(token);
                        }
                    }
                    break;
                case xslLexer_1.XSLTokenLevelState.xslElementName:
                    // this is xslt or schematron element
                    pendingTemplateParamErrors = [];
                    tagElementName = xsltTokenDiagnostics_1.XsltTokenDiagnostics.getTextForToken(lineNumber, token, document);
                    const isXsltElementName = tagElementName.startsWith('xsl:');
                    const isSchElementName = tagElementName.startsWith('sch:');
                    const lookupElementName = isSchematron && !isXsltElementName && !isSchElementName ? 'sch:' + tagElementName : tagElementName;
                    const realSchemaQuery = xsltSchemaQuery && tagElementName.startsWith('xsl:') ? xsltSchemaQuery : schemaQuery;
                    if (tagType === xsltTokenDiagnostics_1.TagType.Start) {
                        if (tagElementName === 'xsl:iterate') {
                            currentXSLTIterateParams.push([]);
                        }
                        tagType = (xslVariable.indexOf(tagElementName) > -1) ? xsltTokenDiagnostics_1.TagType.XSLTvar : xsltTokenDiagnostics_1.TagType.XSLTstart;
                        let xsltToken = token;
                        xsltToken['tagType'] = tagType;
                        startTagToken = token;
                        if (!includeOrImport && tagType !== xsltTokenDiagnostics_1.TagType.XSLTvar && elementStack.length === 1) {
                            includeOrImport = tagElementName === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslImport || tagElementName === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslInclude;
                        }
                        if (!onRootStartTag && elementStack.length === 0) {
                            token['error'] = xpLexer_1.ErrorType.MultiRoot;
                            token['value'] = tagElementName;
                            problemTokens.push(token);
                        }
                    }
                    break;
                case xslLexer_1.XSLTokenLevelState.elementName:
                    tagElementName = xsltTokenDiagnostics_1.XsltTokenDiagnostics.getTextForToken(lineNumber, token, document);
                    if (tagType === xsltTokenDiagnostics_1.TagType.Start) {
                        tagType = xsltTokenDiagnostics_1.TagType.XMLstart;
                        startTagToken = token;
                        if (!onRootStartTag && elementStack.length === 0) {
                            token['error'] = xpLexer_1.ErrorType.MultiRoot;
                            token['value'] = tagElementName;
                            problemTokens.push(token);
                        }
                    }
                    break;
                case xslLexer_1.XSLTokenLevelState.xmlPunctuation:
                    switch (xmlCharType) {
                        case xslLexer_1.XMLCharState.lSt:
                            tagAttributeNames = [];
                            tagAttributeSymbols = [];
                            tagXmlnsNames = [];
                            tagIdentifierName = '';
                            variableData = null;
                            tagElementName = '';
                            tagExcludeResultPrefixes = null;
                            tagType = xsltTokenDiagnostics_1.TagType.Start;
                            break;
                        case xslLexer_1.XMLCharState.rStNoAtt:
                        case xslLexer_1.XMLCharState.rSt:
                        case xslLexer_1.XMLCharState.rSelfCt:
                        case xslLexer_1.XMLCharState.rSelfCtNoAtt:
                            // start-tag ended, we're now within the new element scope:
                            if (docType === xslLexer_1.DocumentTypes.XSLT && onRootStartTag) {
                                rootXmlnsBindings.forEach((prefixNsPair) => {
                                    let pfx = prefixNsPair[0];
                                    let namespaceURI = prefixNsPair[1];
                                    let xsltType = functionData_1.FunctionData.namespaces.get(namespaceURI);
                                    if (xsltType !== undefined) {
                                        xsltPrefixesToURIs.set(pfx, xsltType);
                                    }
                                });
                                if (xsltPrefixesToURIs.get('xsl') !== functionData_1.XSLTnamespaces.XSLT) {
                                    if (startTagToken !== null) {
                                        startTagToken['error'] = xpLexer_1.ErrorType.XSLTNamesapce;
                                        problemTokens.push(startTagToken);
                                    }
                                }
                            }
                            onRootStartTag = false;
                            let orginalPrefixes = inheritedPrefixes.slice();
                            let problem = false;
                            if (tagExcludeResultPrefixes) {
                                let missingPrefix;
                                if (!(tagExcludeResultPrefixes.prefixes.length === 1 && tagExcludeResultPrefixes.prefixes[0] === '#all')) {
                                    missingPrefix = tagExcludeResultPrefixes.prefixes.find((pfx) => {
                                        if (pfx !== '#default' && inheritedPrefixes.indexOf(pfx) < 0)
                                            return pfx;
                                    });
                                }
                                if (missingPrefix) {
                                    let xToken = tagExcludeResultPrefixes.token;
                                    xToken['error'] = xpLexer_1.ErrorType.MissingPrefixInList;
                                    xToken.value = missingPrefix;
                                    problemTokens.push(tagExcludeResultPrefixes.token);
                                    problem = true;
                                }
                            }
                            const attrValType = tagElementName.startsWith('xsl:') ? xsltTokenDiagnostics_1.ValidationType.XSLTAttribute : xsltTokenDiagnostics_1.ValidationType.XMLAttribute;
                            if (xmlCharType === xslLexer_1.XMLCharState.rStNoAtt || xmlCharType === xslLexer_1.XMLCharState.rSt) {
                                // on a start tag
                                if (tagElementName === 'xsl:accumulator') {
                                    inScopeVariablesList.push({ token: token, name: 'value' });
                                }
                                else if (tagElementName === 'xsl:catch') {
                                    xsltTokenDiagnostics_1.XsltTokenDiagnostics.xsltCatchVariables.forEach((catchVar) => {
                                        inScopeVariablesList.push({ token: token, name: catchVar });
                                    });
                                }
                                let inheritedPrefixesCopy = inheritedPrefixes.slice();
                                // if top-level element add global variables - these include following variables also:
                                let newVariablesList = elementStack.length === 0 ? globalVariableData : inScopeVariablesList;
                                const stackElementChildren = isSchematron ? tagElementChildren : attrValType === xsltTokenDiagnostics_1.ValidationType.XMLAttribute && elementStack.length > 0 ? elementStack[elementStack.length - 1].expectedChildElements : tagElementChildren;
                                //let newVariablesList = inScopeVariablesList;
                                const childSymbols = xsltTokenDiagnostics_1.XsltTokenDiagnostics.initChildrenSymbols(tagAttributeSymbols);
                                if (variableData !== null) {
                                    if (elementStack.length > 1) {
                                        xsltVariableDeclarations.push(variableData.token);
                                    }
                                    if (startTagToken) {
                                        // if a top-level element, use global variables instad of inScopeVariablesList;
                                        elementStack.push({
                                            namespacePrefixes: inheritedPrefixesCopy, currentVariable: variableData, variables: newVariablesList,
                                            symbolName: tagElementName, symbolID: tagIdentifierName, identifierToken: startTagToken, childSymbols: childSymbols, expectedChildElements: stackElementChildren
                                        });
                                    }
                                }
                                else if (startTagToken) {
                                    elementStack.push({ namespacePrefixes: inheritedPrefixesCopy, variables: newVariablesList, symbolName: tagElementName, symbolID: tagIdentifierName, identifierToken: startTagToken, childSymbols: childSymbols, expectedChildElements: stackElementChildren });
                                }
                                inScopeVariablesList = [];
                                newVariablesList = [];
                                tagType = xsltTokenDiagnostics_1.TagType.NonStart;
                            }
                            else {
                                // self-closed tag: xmlns declarations on this are no longer in scope
                                inheritedPrefixes = orginalPrefixes;
                                if (variableData !== null) {
                                    if (elementStack.length > 1) {
                                        if (docType === xslLexer_1.DocumentTypes.DCP) {
                                            importedGlobalVarNames.push(variableData.name);
                                            globalVariableData.push(variableData);
                                        }
                                        else {
                                            inScopeVariablesList.push(variableData);
                                        }
                                        xsltVariableDeclarations.push(variableData.token);
                                    }
                                    else {
                                        inScopeVariablesList = [];
                                    }
                                }
                            }
                            break;
                        case xslLexer_1.XMLCharState.rCt:
                            // end of an element close-tag:
                            if (elementStack.length > 0) {
                                let poppedData = elementStack.pop();
                                if (tagElementName === 'xsl:iterate' && currentXSLTIterateParams.length > 0) {
                                    currentXSLTIterateParams.pop();
                                }
                                if (poppedData) {
                                    inheritedPrefixes = poppedData.namespacePrefixes.slice();
                                    inScopeVariablesList = (poppedData) ? poppedData.variables : [];
                                    if (poppedData.currentVariable) {
                                        if (docType === xslLexer_1.DocumentTypes.DCP) {
                                            importedGlobalVarNames.push(poppedData.currentVariable.name);
                                            globalVariableData.push(poppedData.currentVariable);
                                        }
                                        else if (elementStack.length > 1) {
                                            // reset inscope variables - unless at global-variable stack-level
                                            inScopeVariablesList.push(poppedData.currentVariable);
                                        }
                                    }
                                }
                            }
                            break;
                    }
                    break;
                case xslLexer_1.XSLTokenLevelState.attributeName:
                case xslLexer_1.XSLTokenLevelState.xmlnsName:
                    rootXmlnsName = null;
                    let attNameText = xsltTokenDiagnostics_1.XsltTokenDiagnostics.getTextForToken(lineNumber, token, document);
                    const problemReported = false;
                    if (!problemReported) {
                        if (xmlTokenType === xslLexer_1.XSLTokenLevelState.xmlnsName) {
                            tagXmlnsNames.push(attNameText);
                            if (attNameText.length > 6) {
                                let prefix = attNameText.substring(6);
                                if (inheritedPrefixes.indexOf(prefix) < 0) {
                                    inheritedPrefixes.push(prefix);
                                }
                                if (prefix === 'ixsl') {
                                    if (schemaQuery) {
                                        schemaQuery.useIxsl = true;
                                    }
                                    if (xsltSchemaQuery) {
                                        xsltSchemaQuery.useIxsl = true;
                                    }
                                }
                            }
                            if (onRootStartTag) {
                                rootXmlnsName = attNameText;
                            }
                        }
                        else {
                            tagAttributeSymbols.push(xsltTokenDiagnostics_1.XsltTokenDiagnostics.createSymbolForAttribute(token, attNameText));
                            tagAttributeNames.push(attNameText);
                        }
                    }
                    if (tagType === xsltTokenDiagnostics_1.TagType.XSLTvar) {
                        attType = attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslNameAtt ? xsltTokenDiagnostics_1.AttributeType.Variable : xsltTokenDiagnostics_1.AttributeType.None;
                    }
                    else if (tagType === xsltTokenDiagnostics_1.TagType.XSLTstart) {
                        if (docType === xslLexer_1.DocumentTypes.DCP && (attNameText === 'parameterRef' || attNameText === 'if' || attNameText === 'unless')) {
                            attType = xsltTokenDiagnostics_1.AttributeType.VariableRef;
                        }
                        else if (attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslNameAtt) {
                            attType = xsltTokenDiagnostics_1.AttributeType.InstructionName;
                        }
                        else if (attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslModeAtt) {
                            attType = xsltTokenDiagnostics_1.AttributeType.InstructionMode;
                        }
                        else if (attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.useAttSet) {
                            attType = xsltTokenDiagnostics_1.AttributeType.UseAttributeSets;
                        }
                        else if (attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.excludePrefixes || attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslExcludePrefixes) {
                            attType = xsltTokenDiagnostics_1.AttributeType.ExcludeResultPrefixes;
                        }
                        else {
                            attType = xsltTokenDiagnostics_1.AttributeType.None;
                        }
                    }
                    else if (attNameText === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xslUseAttSet) {
                        attType = xsltTokenDiagnostics_1.AttributeType.UseAttributeSets;
                    }
                    break;
                case xslLexer_1.XSLTokenLevelState.attributeValue:
                    let fullVariableName = xsltTokenDiagnostics_1.XsltTokenDiagnostics.getTextForToken(lineNumber, token, document);
                    if (tagAttributeSymbols.length > 0) {
                        if (fullVariableName.length !== 1) {
                            tagAttributeSymbols[tagAttributeSymbols.length - 1].detail = fullVariableName;
                        }
                        else {
                            tagAttributeSymbols[tagAttributeSymbols.length - 1].kind = vscode.SymbolKind.Event;
                        }
                    }
                    let variableName = fullVariableName.substring(1, fullVariableName.length - 1);
                    let hasProblem = false;
                    if (rootXmlnsName !== null) {
                        let prefix = rootXmlnsName.length === 5 ? '' : rootXmlnsName.substr(6);
                        rootXmlnsBindings.push([prefix, variableName]);
                    }
                    switch (attType) {
                        case xsltTokenDiagnostics_1.AttributeType.Variable:
                            tagIdentifierName = variableName;
                            if (elementStack.length > 2) {
                                let parentElemmentName = elementStack[elementStack.length - 1].symbolName;
                                if (parentElemmentName === 'xsl:iterate') {
                                    currentXSLTIterateParams[currentXSLTIterateParams.length - 1].push(variableName);
                                }
                            }
                            variableData = { token: token, name: variableName };
                            break;
                        case xsltTokenDiagnostics_1.AttributeType.VariableRef:
                            let unResolvedToken = xsltTokenDiagnostics_1.XsltTokenDiagnostics.resolveXPathVariableReference('', document, importedGlobalVarNames, token, xpathVariableCurrentlyBeingDefined, inScopeXPathVariablesList, xpathStack, inScopeVariablesList, elementStack);
                            if (unResolvedToken !== null) {
                                unresolvedXsltVariableReferences.push(unResolvedToken);
                            }
                            break;
                        case xsltTokenDiagnostics_1.AttributeType.InstructionName:
                            let slashPos = variableName.lastIndexOf('/');
                            if (slashPos > 0) {
                                // package name may be URI
                                variableName = variableName.substring(slashPos + 1);
                            }
                            tagIdentifierName = variableName;
                            break;
                        case xsltTokenDiagnostics_1.AttributeType.InstructionMode:
                            if (tagIdentifierName === '') {
                                tagIdentifierName = variableName;
                            }
                            break;
                        case xsltTokenDiagnostics_1.AttributeType.ExcludeResultPrefixes:
                            let excludePrefixes = variableName.split(/\s+/);
                            tagExcludeResultPrefixes = { token: token, prefixes: excludePrefixes };
                            break;
                        case xsltTokenDiagnostics_1.AttributeType.None:
                            if (prevToken && prevToken.length === 1 && prevToken.tokenType === xsltTokenDiagnostics_1.XsltTokenDiagnostics.xsltStartTokenNumber + xslLexer_1.XSLTokenLevelState.attributeValue) {
                                token['error'] = xpLexer_1.ErrorType.XPathEmpty;
                            }
                            break;
                    }
                    if (!hasProblem && attType === xsltTokenDiagnostics_1.AttributeType.UseAttributeSets) {
                        if (globalAttributeSetNames.indexOf(variableName) < 0 && variableName !== 'xsl:original') {
                            token['error'] = xpLexer_1.ErrorType.AttributeSetUnresolved;
                            token.value = variableName;
                            problemTokens.push(token);
                            hasProblem = true;
                        }
                    }
                    if (!hasProblem && attType === xsltTokenDiagnostics_1.AttributeType.InstructionName && tagElementName === 'xsl:call-template') {
                        if (!namedTemplates.get(variableName)) {
                            token['error'] = xpLexer_1.ErrorType.TemplateNameUnresolved;
                            token.value = variableName;
                            problemTokens.push(token);
                            hasProblem = true;
                        }
                    }
                    if (!hasProblem && attType === xsltTokenDiagnostics_1.AttributeType.InstructionName && tagElementName === 'xsl:function') {
                        if (!variableName.includes(':')) {
                            token['error'] = xpLexer_1.ErrorType.XSLTFunctionNamePrefix;
                            token.value = variableName;
                            problemTokens.push(token);
                            hasProblem = true;
                        }
                    }
                    if (!hasProblem && attType === xsltTokenDiagnostics_1.AttributeType.InstructionMode && tagElementName === 'xsl:apply-templates') {
                        if (globalModes.indexOf(variableName) < 0) {
                            token['error'] = xpLexer_1.ErrorType.TemplateModeUnresolved;
                            token.value = variableName;
                            problemTokens.push(token);
                            hasProblem = true;
                        }
                    }
                    if (!hasProblem && attType === xsltTokenDiagnostics_1.AttributeType.InstructionName && elementStack.length > 0 && tagElementName === 'xsl:with-param') {
                        let callTemplateName = elementStack[elementStack.length - 1].symbolID;
                        let templateParams = namedTemplates.get(callTemplateName);
                        if (templateParams) {
                            if ((templateParams === null || templateParams === void 0 ? void 0 : templateParams.indexOf(variableName)) < 0) {
                                token['error'] = xpLexer_1.ErrorType.MissingTemplateParam;
                                token.value = `${callTemplateName}#${variableName}`;
                                pendingTemplateParamErrors.push(token);
                            }
                        }
                        else if (currentXSLTIterateParams.length > 0 && elementStack.length > 2 && elementStack[elementStack.length - 1].symbolName === 'xsl:next-iteration') {
                            const params = currentXSLTIterateParams[currentXSLTIterateParams.length - 1];
                            if (params.indexOf(variableName) < 0) {
                                token['error'] = xpLexer_1.ErrorType.IterateParamInvalid;
                                token.value = variableName;
                                problemTokens.push(token);
                                hasProblem = true;
                            }
                        }
                    }
                    attType = xsltTokenDiagnostics_1.AttributeType.None;
                    break;
            }
        }
        else {
            let xpathCharType = token.charType;
            let xpathTokenType = token.tokenType;
            switch (xpathTokenType) {
                case xpLexer_1.TokenLevelState.string:
                    if (xpathStack.length > 0) {
                        let xp = xpathStack[xpathStack.length - 1];
                        if (xp.functionArity === 0 && (((_a = xp.function) === null || _a === void 0 ? void 0 : _a.value) === 'key' || ((_b = xp.function) === null || _b === void 0 ? void 0 : _b.value.startsWith('accumulator-')))) {
                            let keyVal = token.value.substring(1, token.value.length - 1);
                            if (xp.function.value === 'key') {
                                if (globalKeys.indexOf(keyVal) < 0) {
                                    token['error'] = xpLexer_1.ErrorType.XSLTKeyUnresolved;
                                    problemTokens.push(token);
                                }
                            }
                            else if (globalAccumulatorNames.indexOf(keyVal) < 0) {
                                token['error'] = xpLexer_1.ErrorType.AccumulatorNameUnresolved;
                                problemTokens.push(token);
                            }
                        }
                    }
                    break;
                case xpLexer_1.TokenLevelState.variable:
                    if ((preXPathVariable && !xpathVariableCurrentlyBeingDefined) || anonymousFunctionParams) {
                        let fullVariableName = token.value;
                        let currentVariable = { token: token, name: fullVariableName.substring(1) };
                        if (anonymousFunctionParams) {
                            anonymousFunctionParamList.push(currentVariable);
                            xsltVariableDeclarations.push(token);
                        }
                        else {
                            inScopeXPathVariablesList.push(currentVariable);
                            xpathVariableCurrentlyBeingDefined = true;
                            xsltVariableDeclarations.push(token);
                        }
                    }
                    else {
                        let prefixEnd = token.value.indexOf(':');
                        if (prefixEnd !== -1) {
                            let prefix = token.value.substring(1, prefixEnd);
                            if (inheritedPrefixes.indexOf(prefix) === -1) {
                                token['error'] = xpLexer_1.ErrorType.XPathPrefix;
                                problemTokens.push(token);
                            }
                        }
                        // don't include any current pending variable declarations when resolving
                        let globalVarName = null;
                        if (tagType === xsltTokenDiagnostics_1.TagType.XSLTvar && elementStack.length === 1) {
                            globalVarName = tagIdentifierName;
                        }
                        let unResolvedToken = xsltTokenDiagnostics_1.XsltTokenDiagnostics.resolveXPathVariableReference(globalVarName, document, importedGlobalVarNames, token, xpathVariableCurrentlyBeingDefined, inScopeXPathVariablesList, xpathStack, inScopeVariablesList, elementStack);
                        if (unResolvedToken !== null) {
                            unresolvedXsltVariableReferences.push(unResolvedToken);
                        }
                    }
                    break;
                case xpLexer_1.TokenLevelState.complexExpression:
                    let valueText = token.value;
                    switch (valueText) {
                        case 'if':
                            ifThenStack.push(token);
                            break;
                        case 'every':
                        case 'for':
                        case 'let':
                        case 'some':
                            preXPathVariable = true;
                            xpathVariableCurrentlyBeingDefined = false;
                            xpathStack.push({ token: token, variables: inScopeXPathVariablesList.slice(), preXPathVariable: preXPathVariable, xpathVariableCurrentlyBeingDefined: xpathVariableCurrentlyBeingDefined, isRangeVar: true });
                            break;
                        case 'then':
                            if (ifThenStack.length > 0) {
                                ifThenStack.pop();
                            }
                            else {
                                token.error = xpLexer_1.ErrorType.XPathUnexpected;
                                problemTokens.push(token);
                            }
                            xpathStack.push({ token: token, variables: inScopeXPathVariablesList.slice(), preXPathVariable: preXPathVariable, xpathVariableCurrentlyBeingDefined: xpathVariableCurrentlyBeingDefined });
                            inScopeXPathVariablesList = [];
                            break;
                        case 'return':
                        case 'satisfies':
                        case 'else':
                            let tokenValBeforeDelete = xpathStack.length > 0 ? xpathStack[xpathStack.length - 1].token.value : '';
                            if (xpathStack.length > 1) {
                                let deleteCount = 0;
                                for (let i = xpathStack.length - 1; i > -1; i--) {
                                    const sv = xpathStack[i].token.value;
                                    if (sv === 'return' || sv === 'else' || sv === 'satisfies') {
                                        deleteCount++;
                                    }
                                    else {
                                        break;
                                    }
                                }
                                if (deleteCount > 0) {
                                    xpathStack.splice(xpathStack.length - deleteCount);
                                }
                            }
                            if (xpathStack.length > 0) {
                                let peekedStack = xpathStack[xpathStack.length - 1];
                                if (peekedStack) {
                                    if (valueText === 'else') {
                                        preXPathVariable = peekedStack.preXPathVariable;
                                    }
                                    else {
                                        // todo: if after a return AND a ',' prePathVariable = true; see $pos := $c.
                                        preXPathVariable = false;
                                    }
                                    xpathVariableCurrentlyBeingDefined = peekedStack.xpathVariableCurrentlyBeingDefined;
                                    peekedStack.token = token;
                                }
                                else {
                                    inScopeXPathVariablesList = [];
                                    preXPathVariable = false;
                                    xpathVariableCurrentlyBeingDefined = false;
                                }
                            }
                            break;
                    }
                    break;
                case xpLexer_1.TokenLevelState.mapKey:
                    if (!(prevToken && prevToken.tokenType === xpLexer_1.TokenLevelState.operator
                        && (prevToken.value === ',' || prevToken.value === '{'))) {
                        token['error'] = xpLexer_1.ErrorType.XPathUnexpected;
                        problemTokens.push(token);
                    }
                    break;
                case xpLexer_1.TokenLevelState.operator:
                    let isXPathError = false;
                    let tv = token.value;
                    // start checks
                    let stackItem = xpathStack.length > 0 ? xpathStack[xpathStack.length - 1] : undefined;
                    const sv = stackItem === null || stackItem === void 0 ? void 0 : stackItem.token.value;
                    const tokenIsComma = tv === ',';
                    const popStackLaterForComma = sv && tokenIsComma && (sv === 'return' || sv === 'else' || sv === 'satisfies');
                    if (popStackLaterForComma && xpathStack.length > 1) {
                        stackItem = xpathStack[xpathStack.length - 2];
                    }
                    if (stackItem && stackItem.curlyBraceType === xsltTokenDiagnostics_1.CurlyBraceType.Map) {
                        if (tokenIsComma) {
                            if (stackItem.awaitingMapKey) {
                                isXPathError = true;
                            }
                            else {
                                stackItem.awaitingMapKey = true;
                            }
                        }
                        else if (tv === '}' && stackItem.awaitingMapKey) {
                            isXPathError = true;
                        }
                    }
                    if ((prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.complexExpression) {
                        let currCharType = token.charType;
                        if (currCharType === xpLexer_1.CharLevelState.rB || currCharType === xpLexer_1.CharLevelState.rBr || currCharType === xpLexer_1.CharLevelState.rPr) {
                            // do nothing
                        }
                        else if (tokenIsComma) {
                            // do nothing								
                        }
                    }
                    else if (prevToken && tv === ':') {
                        if (stackItem && stackItem.curlyBraceType === xsltTokenDiagnostics_1.CurlyBraceType.Map) {
                            if (stackItem.awaitingMapKey) {
                                stackItem.awaitingMapKey = false;
                            }
                        }
                    }
                    // end checks
                    let functionToken = null;
                    switch (xpathCharType) {
                        case xpLexer_1.CharLevelState.lBr:
                            let curlyBraceType = xsltTokenDiagnostics_1.CurlyBraceType.None;
                            if (prevToken && prevToken.tokenType === xpLexer_1.TokenLevelState.operator) {
                                if (prevToken.value === 'map') {
                                    curlyBraceType = xsltTokenDiagnostics_1.CurlyBraceType.Map;
                                }
                                else if (prevToken.value === 'array') {
                                    curlyBraceType = xsltTokenDiagnostics_1.CurlyBraceType.Array;
                                }
                            }
                            const stackItem = { token: token, variables: inScopeXPathVariablesList, preXPathVariable: preXPathVariable, xpathVariableCurrentlyBeingDefined: xpathVariableCurrentlyBeingDefined, curlyBraceType };
                            if (curlyBraceType === xsltTokenDiagnostics_1.CurlyBraceType.Map) {
                                stackItem.awaitingMapKey = true;
                            }
                            xpathStack.push(stackItem);
                            if (anonymousFunctionParams) {
                                // handle case: function($a) {$a + 8} pass params to inside '{...}'				
                                inScopeXPathVariablesList = anonymousFunctionParamList;
                                anonymousFunctionParamList = [];
                                anonymousFunctionParams = false;
                            }
                            else {
                                inScopeXPathVariablesList = [];
                            }
                            preXPathVariable = false;
                            xpathVariableCurrentlyBeingDefined = false;
                            break;
                        case xpLexer_1.CharLevelState.lB:
                            // handle case: function($a)						
                            if (!anonymousFunctionParams && (prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) !== xpLexer_1.TokenLevelState.nodeType) {
                                anonymousFunctionParams = (prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.anonymousFunction;
                            }
                            if ((prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.function) {
                                functionToken = prevToken;
                            }
                            else if ((prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.variable) {
                                // TODO: check arity of variables of type 'function'
                                incrementFunctionArity = false;
                            }
                        // intentionally no-break;	
                        case xpLexer_1.CharLevelState.lPr:
                            let xpathItem = { token: token, variables: inScopeXPathVariablesList, preXPathVariable: preXPathVariable, xpathVariableCurrentlyBeingDefined: xpathVariableCurrentlyBeingDefined };
                            if (functionToken) {
                                xpathItem.function = functionToken;
                                if (incrementFunctionArity) {
                                    xpathItem.functionArity = 1;
                                    incrementFunctionArity = false;
                                }
                                else {
                                    xpathItem.functionArity = 0;
                                }
                            }
                            xpathStack.push(xpathItem);
                            preXPathVariable = false;
                            inScopeXPathVariablesList = [];
                            xpathVariableCurrentlyBeingDefined = false;
                            break;
                        case xpLexer_1.CharLevelState.rB:
                        case xpLexer_1.CharLevelState.rPr:
                        case xpLexer_1.CharLevelState.rBr:
                            if (xpathStack.length > 1) {
                                let deleteCount = 0;
                                for (let i = xpathStack.length - 1; i > -1; i--) {
                                    const sv = xpathStack[i].token.value;
                                    if (sv === 'return' || sv === 'else' || sv === 'satisfies') {
                                        deleteCount++;
                                    }
                                    else {
                                        break;
                                    }
                                }
                                if (deleteCount > 0) {
                                    xpathStack.splice(xpathStack.length - deleteCount);
                                }
                            }
                            if (xpathStack.length > 0) {
                                let poppedData = xpathStack.pop();
                                if (poppedData) {
                                    if (poppedData.token.value === 'then') {
                                        poppedData.token['error'] = xpLexer_1.ErrorType.BracketNesting;
                                        problemTokens.push(poppedData.token);
                                    }
                                    inScopeXPathVariablesList = poppedData.variables;
                                    preXPathVariable = poppedData.preXPathVariable;
                                    xpathVariableCurrentlyBeingDefined = poppedData.xpathVariableCurrentlyBeingDefined;
                                    if (poppedData.function && poppedData.functionArity !== undefined) {
                                        if ((prevToken === null || prevToken === void 0 ? void 0 : prevToken.charType) !== xpLexer_1.CharLevelState.lB) {
                                            if (poppedData.functionArity !== undefined) {
                                                poppedData.functionArity++;
                                            }
                                        }
                                        if (seekInstruction.type === xslLexer_1.GlobalInstructionType.Function) {
                                            let functionName;
                                            let arity = poppedData.functionArity;
                                            const fnToken = poppedData.function;
                                            if (arity === undefined) {
                                                let parts = fnToken.value.split('#');
                                                arity = Number.parseInt(parts[1]);
                                                functionName = parts[0];
                                            }
                                            else {
                                                functionName = fnToken.value;
                                            }
                                            if (functionName === seekInstruction.name && arity === seekInstruction.idNumber) {
                                                referenceTokens.push(fnToken);
                                            }
                                        }
                                    }
                                }
                                else {
                                    inScopeXPathVariablesList = [];
                                    preXPathVariable = false;
                                    xpathVariableCurrentlyBeingDefined = false;
                                }
                            }
                            if (token.error && !isXPathError) {
                                // any error should already have been added by lexer:
                                problemTokens.push(token);
                            }
                            break;
                        case xpLexer_1.CharLevelState.sep:
                            if (token.value === ',') {
                                if (xpathStack.length > 0) {
                                    let xp = xpathStack[xpathStack.length - 1];
                                    if (xp.functionArity !== undefined) {
                                        xp.functionArity++;
                                    }
                                    if (xp.isRangeVar) {
                                        preXPathVariable = xp.preXPathVariable;
                                    }
                                    let nonBracketedThen = -1;
                                    for (let i = xpathStack.length - 1; i > -1; i--) {
                                        const xpathItem = xpathStack[i].token;
                                        const val = xpathItem.value;
                                        if (!(val === 'return' || val === 'else' || val === 'satisfies' || val === 'then')) {
                                            break;
                                        }
                                        else if (val === 'then') {
                                            nonBracketedThen = i;
                                        }
                                    }
                                    if (nonBracketedThen > -1) {
                                        //xpathStack.splice(nonBracketedThen, 1);
                                        token['error'] = xpLexer_1.ErrorType.ExpectedElseAfterThen;
                                        problemTokens.push(token);
                                    }
                                    const sv = xp.token.value;
                                    if (sv === 'return' || sv === 'else' || sv === 'satisfies') {
                                        let poppedData = xpathStack.pop();
                                        if (poppedData) {
                                            inScopeXPathVariablesList = poppedData.variables;
                                            if (sv === 'else') {
                                                preXPathVariable = poppedData.preXPathVariable;
                                            }
                                            else {
                                                // todo: if after a return AND a ',' prePathVariable = true; see $pos := $c.
                                                preXPathVariable = false;
                                            }
                                            xpathVariableCurrentlyBeingDefined = false;
                                        }
                                    }
                                }
                                xpathVariableCurrentlyBeingDefined = false;
                            }
                            break;
                        case xpLexer_1.CharLevelState.dSep:
                            console.log('dSap');
                            const isEmptyBracketsToken = token.value === '()';
                            if (isEmptyBracketsToken && (prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.function) {
                                const fnArity = incrementFunctionArity ? 1 : 0;
                                incrementFunctionArity = false;
                                if (seekInstruction.type === xslLexer_1.GlobalInstructionType.Function) {
                                    const sameArity = fnArity == seekInstruction.idNumber;
                                    console.log({ sameArity });
                                    if (prevToken.value === seekInstruction.name && sameArity) {
                                        console.log({ prevToken });
                                        referenceTokens.push(prevToken);
                                    }
                                }
                            }
                            else if (isEmptyBracketsToken && (prevToken === null || prevToken === void 0 ? void 0 : prevToken.tokenType) === xpLexer_1.TokenLevelState.variable) {
                                // TODO: check arity of variable of type 'function'
                                incrementFunctionArity = false;
                            }
                            else if (token.value === '=>') {
                                incrementFunctionArity = true;
                            }
                            break;
                    }
                    break;
                case xpLexer_1.TokenLevelState.functionNameTest:
                    let { isValid, qFunctionName, fErrorType } = xsltTokenDiagnostics_1.XsltTokenDiagnostics.isValidFunctionName(inheritedPrefixes, xsltPrefixesToURIs, token, importedGlobalFnNames);
                    if (!isValid) {
                        token['error'] = fErrorType;
                        token['value'] = qFunctionName;
                        problemTokens.push(token);
                    }
                    break;
            }
            if (index === lastTokenIndex && !token.error) {
                if (token.tokenType === xpLexer_1.TokenLevelState.operator) {
                    xsltTokenDiagnostics_1.XsltTokenDiagnostics.checkFinalXPathToken(token, allTokens, index, problemTokens);
                }
                if (xpathStack.length > 0 && !token.error) {
                    let disallowedStackItem;
                    for (let index = xpathStack.length - 1; index > -1; index--) {
                        const trailingToken = xpathStack[index].token;
                        const tv = trailingToken.value;
                        const allowedToken = (tv === 'return' || tv === 'else' || tv === 'satisfies');
                        if (!allowedToken) {
                            disallowedStackItem = trailingToken;
                            break;
                        }
                    }
                    if (disallowedStackItem) {
                        disallowedStackItem['error'] = xpLexer_1.ErrorType.BracketNesting;
                        problemTokens.push(disallowedStackItem);
                    }
                }
                if (token.tokenType === xpLexer_1.TokenLevelState.string && !token.error) {
                    xpLexer_1.XPathLexer.checkStringLiteralEnd(token);
                    if (token.error) {
                        problemTokens.push(token);
                    }
                }
            }
        }
        prevToken = token.tokenType === xpLexer_1.TokenLevelState.comment ? prevToken : token;
        if (index === lastTokenIndex) {
            // xml is not well-nested if items still on the stack at the end
            // but report errors and try to keep some part of the tree:
            if (token.tokenType === xpLexer_1.TokenLevelState.complexExpression) {
                token['error'] = xpLexer_1.ErrorType.XPathAwaiting;
                problemTokens.push(token);
            }
            if (elementStack.length > 0) {
                let usedtoken = false;
                while (elementStack.length > 0) {
                    let poppedData = elementStack.pop();
                    let endToken;
                    if (poppedData) {
                        if (usedtoken) {
                            // use final token as we don't know what the end token is 
                            // but reduce lendth by one on each iteration - so its well nested
                            endToken = token;
                            endToken.length = endToken.length - 1;
                        }
                        else {
                            endToken = token;
                            usedtoken = true;
                        }
                        let errorToken = Object.assign({}, poppedData.identifierToken);
                        errorToken['error'] = xpLexer_1.ErrorType.ElementNesting;
                        problemTokens.push(errorToken);
                    }
                }
            }
        }
    });
    return referenceTokens;
};
//# sourceMappingURL=xsltReferenceProvider.js.map