"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const xpLexer_1 = require("./xpLexer");
class XPathDocumentChangeHandler {
    onDocumentChange(e) {
        return __awaiter(this, void 0, void 0, function* () {
            let activeChange = e.contentChanges[0];
            if (!activeChange) {
                return;
            }
            let doTrigger = false;
            const docText = e.document.getText();
            let precededByWS = false;
            let followsSeparator = false;
            const pos = activeChange.rangeOffset - 1;
            let wsEndPos = -1;
            let keywordPos = -1;
            let keyword;
            let separatorChar;
            const char = activeChange.text;
            let charIsWs = (char === ' ' || char === '\t' || char === '\n');
            for (let x = pos; x > -1; x--) {
                const char = docText.charAt(x);
                if (char === ' ' || char === '\t' || char === '\n') {
                    precededByWS = true;
                    if (keywordPos !== -1) {
                        break;
                    }
                }
                else if (xpLexer_1.Data.anySeps.indexOf(char) !== -1) {
                    followsSeparator = true;
                    separatorChar = char;
                    break;
                }
                else if (charIsWs) {
                    if (wsEndPos === -1) {
                        wsEndPos = x + 1;
                    }
                    keywordPos = x;
                }
                else {
                    break;
                }
            }
            keyword = keywordPos !== -1 ? docText.substring(keywordPos, (wsEndPos)) : undefined;
            if (charIsWs) {
                doTrigger = followsSeparator || (keyword !== undefined && xpLexer_1.Data.triggerWords.indexOf(keyword) !== -1);
            }
            else if (char === '$' || char === '/' || char === '[' || char === '?' || char === '@') {
                doTrigger = true;
            }
            else if (followsSeparator && (char === ':' && separatorChar === ':')) {
                doTrigger = true;
            }
            else {
                doTrigger = precededByWS && followsSeparator;
            }
            //console.log('doTrigger', doTrigger);
            if (doTrigger) {
                setTimeout(() => {
                    vscode.commands.executeCommand('editor.action.triggerSuggest');
                }, 10);
                return;
            }
        });
    }
}
exports.XPathDocumentChangeHandler = XPathDocumentChangeHandler;
//# sourceMappingURL=xpathDocumentChangeHandler.js.map